/*
 *
 * nucleo/core/carbon/cTimeKeeper.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <nucleo/core/carbon/cTimeKeeper.H>
#include <nucleo/utils/TimeUtils.H>

#include <sys/time.h>

#include <iostream>

namespace nucleo {

  // -----------------------------------------------------------------------------

  void
  cTimeKeeper::_callback(EventLoopTimerRef timerId, void *data){
    // std::cerr << " cTimeKeeper::_callback " << std::endl ;

    cTimeKeeper *obj = (cTimeKeeper*)data ;

    obj->notifyObservers() ;

    if (obj->_repeat) {
	 obj->_state = TRIGGERED_AND_ARMED ;
	 obj->_epoch = getCurrentTimeAsMillisecs() + obj->_millisecs ;
    } else {
	 RemoveEventLoopTimer(obj->_timerId) ;
	 obj->_state = TRIGGERED ;
    }
  }

  cTimeKeeper::~cTimeKeeper() {
    disarm() ;
  }

  void
  cTimeKeeper::arm(unsigned long milliseconds, bool repeat) {
    disarm() ;

    EventTimerInterval eti = milliseconds*kEventDurationMillisecond ;

    if (!eti) eti = kEventDurationMicrosecond ;

    InstallEventLoopTimer(GetMainEventLoop(),
					 eti, (repeat ? eti : 0),
					 NewEventLoopTimerUPP(_callback), this, &_timerId) ;

    _millisecs = milliseconds ;
    _epoch = getCurrentTimeAsMillisecs() + milliseconds ;
    _repeat = repeat ;
    _state = ARMED ;
  }

  long
  cTimeKeeper::getTimeLeft(void) {
    if (_state==DISARMED) return -1 ;
    if (_state==TRIGGERED) return 0 ;
    long timeleft = _epoch - getCurrentTimeAsMillisecs() ;
    return (timeleft<0) ? 0 : timeleft ;
  }

  void
  cTimeKeeper::disarm(void) {
    if (_state|ARMED) RemoveEventLoopTimer(_timerId) ;
    _state = DISARMED ;
  }

  // -----------------------------------------------------------------------------

}
