/*
 *
 * nucleo/image/encoding/RGBAL.H --
 *
 * Copyright (C) Olivier Chapuis
 *
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <nucleo/image/encoding/RGB565.H>

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


namespace nucleo {

#ifndef CARD32
#define CARD32 unsigned int
#define CARD16 unsigned short
#endif


  //------------------------------------------------------
  // ARGB -> RGB565

void ARGB2RGB565(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst)
{
	CARD16 *cdst = (CARD16 *) dst;

	for (unsigned int i=w*h; i; --i)
	{
		*cdst = (((src[1] >> 3) & 0x1f) << 11)  |
			(((src[2] >> 2) & 0x3f) << 5)   |
			(((src[3] >> 3) & 0x1f) << 0);
		cdst++;
		src += 4;
	}

	return;
}

 //------------------------------------------------------
 // RGBA -> RGB565

void RGBA2RGB565(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst)
{
	CARD16 *cdst = (CARD16 *) dst;

	for (unsigned int i=w*h; i; --i)
	{
		*cdst = (((src[0] >> 3) & 0x1f) << 11)  |
			(((src[1] >> 2) & 0x3f) << 5)   |
			(((src[2] >> 3) & 0x1f) << 0);
		cdst++;
		src += 4;
	}

	return;
}

 //------------------------------------------------------
 // RGB -> RGB565

void RGB2RGB565(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst)
{
	CARD16 *cdst = (CARD16 *) dst;

	for (unsigned int i=w*h; i; --i)
	{
		*cdst = (((src[0] >> 3) & 0x1f) << 11)  |
			(((src[1] >> 2) & 0x3f) << 5)   |
			(((src[2] >> 3) & 0x1f) << 0);
		cdst++;
		src += 3;
	}

	return;
}

 //------------------------------------------------------
 // L2 -> RGB565

void L2RGB565(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst)
{
	CARD16 *cdst = (CARD16 *) dst;

	for (unsigned int i=w*h; i; --i)
	{
		*cdst = (((*src >> 3) & 0x1f) << 11)  |
			(((*src >> 2) & 0x3f) << 5)   |
			(((*src >> 3) & 0x1f) << 0);
		cdst++;
		src++;
	}
}

 //------------------------------------------------------
 // RGB565 -> RGB

void RGB5652RGB(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst)
{
	CARD16 *csrc = (CARD16 *) dst;

	for (unsigned int i=w*h; i; --i)
	{
		*dst++ = (((*csrc >> 11) & 0x1f) << 3);
		*dst++ = (((*csrc >> 5) & 0x3f) << 2);
		*dst++ = ((*csrc & 0x1f) << 3);
		csrc++;
	}
}

 //------------------------------------------------------
 // RGB565 -> L

void RGB5652L(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst)
{
	CARD16 *csrc = (CARD16 *) dst;

	for (unsigned int i=w*h; i; --i)
	{
		double r,g,b;
		r = (((*csrc >> 11) & 0x1f) << 3);
		g = (((*csrc >> 5) & 0x3f) << 2);
		b = ((*csrc & 0x1f) << 3);
		*dst++ = (unsigned char)(0.11*b + 0.59*g + 0.3*r) ;
		csrc++;
	}
}

 //------------------------------------------------------
 // RGB565 -> RGBA

void RGB5652RGBA(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst)
{
	CARD16 *csrc = (CARD16 *) dst;

	for (unsigned int i=w*h; i; --i)
	{
		*dst++ = ((*csrc >> 11) & 0x1f) << 3;
		*dst++ = ((*csrc >> 5) & 0x3f) << 2;
		*dst++ = (*csrc & 0x1f) << 3;
		*dst++ = 0xff;
		csrc++;
	}
}

 //------------------------------------------------------
 // RGB565 -> ARGB
void RGB5652ARGB(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst)
{
	CARD16 *csrc = (CARD16 *) dst;
	for (unsigned int i=w*h; i; --i)
	{
		*dst++ = 0xff;
		*dst++ = ((*csrc >> 11) & 0x1f) << 3;
		*dst++ = ((*csrc >> 5) & 0x3f) << 2;
		*dst++ = (*csrc & 0x1f) << 3;
		csrc++;
	}
}

}
