/*
 *
 * tests/test-ReactiveEngine.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/core/ReactiveEngine.H>
#include <nucleo/core/FileKeeper.H>
#include <nucleo/core/TimeKeeper.H>

#include <iostream>

using namespace nucleo ;

void
testFileKeeper(void) {
  FileKeeper *file = FileKeeper::create(0, FileKeeper::R) ;
  WatchDog rantanplan(file) ;

  char tmp[1024] ;

  std::cerr << std::endl ;
  std::cerr << "step without timeout: "
		  << ReactiveEngine::step() << std::endl ;
  std::cerr << "input: " << std::flush ; std::cin >> tmp ;

  std::cerr << std::endl ;
  std::cerr << "step with a timeout (4 seconds): " 
		  << ReactiveEngine::step(4*TimeKeeper::second) << std::endl ;
  std::cerr << "input: " << std::flush ; std::cin >> tmp ;
}

void
testTimeKeeper(void) {
  TimeKeeper *timer = TimeKeeper::create() ;
  WatchDog rantanplan(timer) ;

  std::cerr << std::endl ;
  timer->arm(2*TimeKeeper::second) ;
  std::cerr << "step without timeout: "
		  << ReactiveEngine::step() << std::endl ;

  std::cerr << std::endl ;
  timer->arm(2*TimeKeeper::second) ;
  std::cerr << "step with a timeout longer than the timer: " 
		  << ReactiveEngine::step(4*TimeKeeper::second) << std::endl ;

  std::cerr << std::endl ;
  timer->arm(4*TimeKeeper::second) ;
  std::cerr << "step with a timeout shorter than the timer: "
		  << ReactiveEngine::step(2*TimeKeeper::second) << std::endl ;
}

int
main(int argc, char **argv) {
  char *NATIVE = getenv("NUCLEO_USE_NATIVE_ENGINE") ;
  ReactiveEngine::useNativeEngine(!NATIVE || strcmp(NATIVE,"0")) ;

  while (ReactiveEngine::step(1*TimeKeeper::second)) ;
  testFileKeeper() ;

  while (ReactiveEngine::step(1*TimeKeeper::second)) ;
  testTimeKeeper() ;

  return 0 ;
}
